// This file is generated by kconfig_compiler_kf6 from kwintouchscreenscriptsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_KWINTOUCHSCREENSCRIPTSETTINGS_H
#define KWIN_KWINTOUCHSCREENSCRIPTSETTINGS_H

#include "effect/globals.h"

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class KWinTouchScreenScriptSettings : public KConfigSkeleton
{
  public:

    KWinTouchScreenScriptSettings( const QString & ScriptName, QObject *parent = nullptr );
    ~KWinTouchScreenScriptSettings() override;

    /**
      Set TouchBorderActivate
    */
    void setTouchBorderActivate( const QList<int> & v )
    {
      if (!isTouchBorderActivateImmutable())
        mTouchBorderActivate = v;
    }

    /**
      Get TouchBorderActivate
    */
    QList<int> touchBorderActivate() const
    {
      return mTouchBorderActivate;
    }

    /**
      Is TouchBorderActivate Immutable
    */
    bool isTouchBorderActivateImmutable() const
    {
      return isImmutable( QStringLiteral( "TouchBorderActivate" ) );
    }

    /**
      Get TouchBorderActivate default value
    */
    QList<int> defaultTouchBorderActivateValue() const
    {
        return defaultTouchBorderActivateValue_helper();
    }

  protected:
    QString mParamScriptName;

    // Script-$(ScriptName)
    QList<int> mTouchBorderActivate;
    QList<int> defaultTouchBorderActivateValue_helper() const;
};

}

#endif

