// This file was generated by qtwaylandscanner
// source file is tablet-v2.xml

#ifndef QT_WAYLAND_SERVER_TABLET_V2
#define QT_WAYLAND_SERVER_TABLET_V2

#include "wayland-server-core.h"
#include "wayland-tablet-v2-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#include <unistd.h>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_GCC("-Wmissing-field-initializers")
QT_WARNING_DISABLE_CLANG("-Wmissing-field-initializers")

namespace QtWaylandServer {
    class  zwp_tablet_manager_v2
    {
    public:
        zwp_tablet_manager_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_manager_v2(struct ::wl_display *display, int version);
        zwp_tablet_manager_v2(struct ::wl_resource *resource);
        zwp_tablet_manager_v2();

        virtual ~zwp_tablet_manager_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_manager_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_manager_v2 *zwp_tablet_manager_v2_object;
            zwp_tablet_manager_v2 *object() { return zwp_tablet_manager_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


    protected:
        virtual Resource *zwp_tablet_manager_v2_allocate();

        virtual void zwp_tablet_manager_v2_destroy_global();

        virtual void zwp_tablet_manager_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_manager_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_manager_v2_get_tablet_seat(Resource *resource, uint32_t tablet_seat, struct ::wl_resource *seat);
        virtual void zwp_tablet_manager_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_manager_v2_interface m_zwp_tablet_manager_v2_interface;

        static void handle_get_tablet_seat(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t tablet_seat,
            struct ::wl_resource *seat);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_manager_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_seat_v2
    {
    public:
        zwp_tablet_seat_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_seat_v2(struct ::wl_display *display, int version);
        zwp_tablet_seat_v2(struct ::wl_resource *resource);
        zwp_tablet_seat_v2();

        virtual ~zwp_tablet_seat_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_seat_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_seat_v2 *zwp_tablet_seat_v2_object;
            zwp_tablet_seat_v2 *object() { return zwp_tablet_seat_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        void send_tablet_added(struct ::wl_resource *id);
        void send_tablet_added(struct ::wl_resource *resource, struct ::wl_resource *id);
        void send_tool_added(struct ::wl_resource *id);
        void send_tool_added(struct ::wl_resource *resource, struct ::wl_resource *id);
        void send_pad_added(struct ::wl_resource *id);
        void send_pad_added(struct ::wl_resource *resource, struct ::wl_resource *id);

    protected:
        virtual Resource *zwp_tablet_seat_v2_allocate();

        virtual void zwp_tablet_seat_v2_destroy_global();

        virtual void zwp_tablet_seat_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_seat_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_seat_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_seat_v2_interface m_zwp_tablet_seat_v2_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_seat_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_tool_v2
    {
    public:
        zwp_tablet_tool_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_tool_v2(struct ::wl_display *display, int version);
        zwp_tablet_tool_v2(struct ::wl_resource *resource);
        zwp_tablet_tool_v2();

        virtual ~zwp_tablet_tool_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_tool_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_tool_v2 *zwp_tablet_tool_v2_object;
            zwp_tablet_tool_v2 *object() { return zwp_tablet_tool_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum type {
            type_pen = 0x140, // Pen
            type_eraser = 0x141, // Eraser
            type_brush = 0x142, // Brush
            type_pencil = 0x143, // Pencil
            type_airbrush = 0x144, // Airbrush
            type_finger = 0x145, // Finger
            type_mouse = 0x146, // Mouse
            type_lens = 0x147, // Lens
        };

        enum capability {
            capability_tilt = 1, // Tilt axes
            capability_pressure = 2, // Pressure axis
            capability_distance = 3, // Distance axis
            capability_rotation = 4, // Z-rotation axis
            capability_slider = 5, // Slider axis
            capability_wheel = 6, // Wheel axis
        };

        enum button_state {
            button_state_released = 0, // button is not pressed
            button_state_pressed = 1, // button is pressed
        };

        enum error {
            error_role = 0, // given wl_surface has another role
        };

        void send_type(uint32_t tool_type);
        void send_type(struct ::wl_resource *resource, uint32_t tool_type);
        void send_hardware_serial(uint32_t hardware_serial_hi, uint32_t hardware_serial_lo);
        void send_hardware_serial(struct ::wl_resource *resource, uint32_t hardware_serial_hi, uint32_t hardware_serial_lo);
        void send_hardware_id_wacom(uint32_t hardware_id_hi, uint32_t hardware_id_lo);
        void send_hardware_id_wacom(struct ::wl_resource *resource, uint32_t hardware_id_hi, uint32_t hardware_id_lo);
        void send_capability(uint32_t capability);
        void send_capability(struct ::wl_resource *resource, uint32_t capability);
        void send_done();
        void send_done(struct ::wl_resource *resource);
        void send_removed();
        void send_removed(struct ::wl_resource *resource);
        void send_proximity_in(uint32_t serial, struct ::wl_resource *tablet, struct ::wl_resource *surface);
        void send_proximity_in(struct ::wl_resource *resource, uint32_t serial, struct ::wl_resource *tablet, struct ::wl_resource *surface);
        void send_proximity_out();
        void send_proximity_out(struct ::wl_resource *resource);
        void send_down(uint32_t serial);
        void send_down(struct ::wl_resource *resource, uint32_t serial);
        void send_up();
        void send_up(struct ::wl_resource *resource);
        void send_motion(wl_fixed_t x, wl_fixed_t y);
        void send_motion(struct ::wl_resource *resource, wl_fixed_t x, wl_fixed_t y);
        void send_pressure(uint32_t pressure);
        void send_pressure(struct ::wl_resource *resource, uint32_t pressure);
        void send_distance(uint32_t distance);
        void send_distance(struct ::wl_resource *resource, uint32_t distance);
        void send_tilt(wl_fixed_t tilt_x, wl_fixed_t tilt_y);
        void send_tilt(struct ::wl_resource *resource, wl_fixed_t tilt_x, wl_fixed_t tilt_y);
        void send_rotation(wl_fixed_t degrees);
        void send_rotation(struct ::wl_resource *resource, wl_fixed_t degrees);
        void send_slider(int32_t position);
        void send_slider(struct ::wl_resource *resource, int32_t position);
        void send_wheel(wl_fixed_t degrees, int32_t clicks);
        void send_wheel(struct ::wl_resource *resource, wl_fixed_t degrees, int32_t clicks);
        void send_button(uint32_t serial, uint32_t button, uint32_t state);
        void send_button(struct ::wl_resource *resource, uint32_t serial, uint32_t button, uint32_t state);
        void send_frame(uint32_t time);
        void send_frame(struct ::wl_resource *resource, uint32_t time);

    protected:
        virtual Resource *zwp_tablet_tool_v2_allocate();

        virtual void zwp_tablet_tool_v2_destroy_global();

        virtual void zwp_tablet_tool_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_tool_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_tool_v2_set_cursor(Resource *resource, uint32_t serial, struct ::wl_resource *surface, int32_t hotspot_x, int32_t hotspot_y);
        virtual void zwp_tablet_tool_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_tool_v2_interface m_zwp_tablet_tool_v2_interface;

        static void handle_set_cursor(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t serial,
            struct ::wl_resource *surface,
            int32_t hotspot_x,
            int32_t hotspot_y);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_tool_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_v2
    {
    public:
        zwp_tablet_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_v2(struct ::wl_display *display, int version);
        zwp_tablet_v2(struct ::wl_resource *resource);
        zwp_tablet_v2();

        virtual ~zwp_tablet_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_v2 *zwp_tablet_v2_object;
            zwp_tablet_v2 *object() { return zwp_tablet_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum bustype {
            bustype_usb = 3, // USB
            bustype_bluetooth = 5, // Bluetooth
            bustype_virtual = 6, // Virtual
            bustype_serial = 17, // Serial
            bustype_i2c = 24, // I2C
        };

        void send_name(const QString &name);
        void send_name(struct ::wl_resource *resource, const QString &name);
        void send_id(uint32_t vid, uint32_t pid);
        void send_id(struct ::wl_resource *resource, uint32_t vid, uint32_t pid);
        void send_path(const QString &path);
        void send_path(struct ::wl_resource *resource, const QString &path);
        void send_done();
        void send_done(struct ::wl_resource *resource);
        void send_removed();
        void send_removed(struct ::wl_resource *resource);
        void send_bustype(uint32_t bustype);
        void send_bustype(struct ::wl_resource *resource, uint32_t bustype);

    protected:
        virtual Resource *zwp_tablet_v2_allocate();

        virtual void zwp_tablet_v2_destroy_global();

        virtual void zwp_tablet_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_v2_interface m_zwp_tablet_v2_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_pad_ring_v2
    {
    public:
        zwp_tablet_pad_ring_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_pad_ring_v2(struct ::wl_display *display, int version);
        zwp_tablet_pad_ring_v2(struct ::wl_resource *resource);
        zwp_tablet_pad_ring_v2();

        virtual ~zwp_tablet_pad_ring_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_pad_ring_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_pad_ring_v2 *zwp_tablet_pad_ring_v2_object;
            zwp_tablet_pad_ring_v2 *object() { return zwp_tablet_pad_ring_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum source {
            source_finger = 1, // finger
        };

        void send_source(uint32_t source);
        void send_source(struct ::wl_resource *resource, uint32_t source);
        void send_angle(wl_fixed_t degrees);
        void send_angle(struct ::wl_resource *resource, wl_fixed_t degrees);
        void send_stop();
        void send_stop(struct ::wl_resource *resource);
        void send_frame(uint32_t time);
        void send_frame(struct ::wl_resource *resource, uint32_t time);

    protected:
        virtual Resource *zwp_tablet_pad_ring_v2_allocate();

        virtual void zwp_tablet_pad_ring_v2_destroy_global();

        virtual void zwp_tablet_pad_ring_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_pad_ring_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_pad_ring_v2_set_feedback(Resource *resource, const QString &description, uint32_t serial);
        virtual void zwp_tablet_pad_ring_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_pad_ring_v2_interface m_zwp_tablet_pad_ring_v2_interface;

        static void handle_set_feedback(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *description,
            uint32_t serial);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_pad_ring_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_pad_strip_v2
    {
    public:
        zwp_tablet_pad_strip_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_pad_strip_v2(struct ::wl_display *display, int version);
        zwp_tablet_pad_strip_v2(struct ::wl_resource *resource);
        zwp_tablet_pad_strip_v2();

        virtual ~zwp_tablet_pad_strip_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_pad_strip_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_pad_strip_v2 *zwp_tablet_pad_strip_v2_object;
            zwp_tablet_pad_strip_v2 *object() { return zwp_tablet_pad_strip_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum source {
            source_finger = 1, // finger
        };

        void send_source(uint32_t source);
        void send_source(struct ::wl_resource *resource, uint32_t source);
        void send_position(uint32_t position);
        void send_position(struct ::wl_resource *resource, uint32_t position);
        void send_stop();
        void send_stop(struct ::wl_resource *resource);
        void send_frame(uint32_t time);
        void send_frame(struct ::wl_resource *resource, uint32_t time);

    protected:
        virtual Resource *zwp_tablet_pad_strip_v2_allocate();

        virtual void zwp_tablet_pad_strip_v2_destroy_global();

        virtual void zwp_tablet_pad_strip_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_pad_strip_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_pad_strip_v2_set_feedback(Resource *resource, const QString &description, uint32_t serial);
        virtual void zwp_tablet_pad_strip_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_pad_strip_v2_interface m_zwp_tablet_pad_strip_v2_interface;

        static void handle_set_feedback(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *description,
            uint32_t serial);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_pad_strip_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_pad_group_v2
    {
    public:
        zwp_tablet_pad_group_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_pad_group_v2(struct ::wl_display *display, int version);
        zwp_tablet_pad_group_v2(struct ::wl_resource *resource);
        zwp_tablet_pad_group_v2();

        virtual ~zwp_tablet_pad_group_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_pad_group_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_pad_group_v2 *zwp_tablet_pad_group_v2_object;
            zwp_tablet_pad_group_v2 *object() { return zwp_tablet_pad_group_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        void send_buttons(const QByteArray &buttons);
        void send_buttons(struct ::wl_resource *resource, const QByteArray &buttons);
        void send_ring(struct ::wl_resource *ring);
        void send_ring(struct ::wl_resource *resource, struct ::wl_resource *ring);
        void send_strip(struct ::wl_resource *strip);
        void send_strip(struct ::wl_resource *resource, struct ::wl_resource *strip);
        void send_modes(uint32_t modes);
        void send_modes(struct ::wl_resource *resource, uint32_t modes);
        void send_done();
        void send_done(struct ::wl_resource *resource);
        void send_mode_switch(uint32_t time, uint32_t serial, uint32_t mode);
        void send_mode_switch(struct ::wl_resource *resource, uint32_t time, uint32_t serial, uint32_t mode);
        void send_dial(struct ::wl_resource *dial);
        void send_dial(struct ::wl_resource *resource, struct ::wl_resource *dial);

    protected:
        virtual Resource *zwp_tablet_pad_group_v2_allocate();

        virtual void zwp_tablet_pad_group_v2_destroy_global();

        virtual void zwp_tablet_pad_group_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_pad_group_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_pad_group_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_pad_group_v2_interface m_zwp_tablet_pad_group_v2_interface;

        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_pad_group_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_pad_v2
    {
    public:
        zwp_tablet_pad_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_pad_v2(struct ::wl_display *display, int version);
        zwp_tablet_pad_v2(struct ::wl_resource *resource);
        zwp_tablet_pad_v2();

        virtual ~zwp_tablet_pad_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_pad_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_pad_v2 *zwp_tablet_pad_v2_object;
            zwp_tablet_pad_v2 *object() { return zwp_tablet_pad_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        enum button_state {
            button_state_released = 0, // the button is not pressed
            button_state_pressed = 1, // the button is pressed
        };

        void send_group(struct ::wl_resource *pad_group);
        void send_group(struct ::wl_resource *resource, struct ::wl_resource *pad_group);
        void send_path(const QString &path);
        void send_path(struct ::wl_resource *resource, const QString &path);
        void send_buttons(uint32_t buttons);
        void send_buttons(struct ::wl_resource *resource, uint32_t buttons);
        void send_done();
        void send_done(struct ::wl_resource *resource);
        void send_button(uint32_t time, uint32_t button, uint32_t state);
        void send_button(struct ::wl_resource *resource, uint32_t time, uint32_t button, uint32_t state);
        void send_enter(uint32_t serial, struct ::wl_resource *tablet, struct ::wl_resource *surface);
        void send_enter(struct ::wl_resource *resource, uint32_t serial, struct ::wl_resource *tablet, struct ::wl_resource *surface);
        void send_leave(uint32_t serial, struct ::wl_resource *surface);
        void send_leave(struct ::wl_resource *resource, uint32_t serial, struct ::wl_resource *surface);
        void send_removed();
        void send_removed(struct ::wl_resource *resource);

    protected:
        virtual Resource *zwp_tablet_pad_v2_allocate();

        virtual void zwp_tablet_pad_v2_destroy_global();

        virtual void zwp_tablet_pad_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_pad_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_pad_v2_set_feedback(Resource *resource, uint32_t button, const QString &description, uint32_t serial);
        virtual void zwp_tablet_pad_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_pad_v2_interface m_zwp_tablet_pad_v2_interface;

        static void handle_set_feedback(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t button,
            const char *description,
            uint32_t serial);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_pad_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };

    class  zwp_tablet_pad_dial_v2
    {
    public:
        zwp_tablet_pad_dial_v2(struct ::wl_client *client, int id, int version);
        zwp_tablet_pad_dial_v2(struct ::wl_display *display, int version);
        zwp_tablet_pad_dial_v2(struct ::wl_resource *resource);
        zwp_tablet_pad_dial_v2();

        virtual ~zwp_tablet_pad_dial_v2();

        class Resource
        {
        public:
            Resource() : zwp_tablet_pad_dial_v2_object(nullptr), handle(nullptr) {}
            virtual ~Resource() {}

            zwp_tablet_pad_dial_v2 *zwp_tablet_pad_dial_v2_object;
            zwp_tablet_pad_dial_v2 *object() { return zwp_tablet_pad_dial_v2_object; } 
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return wl_resource_get_client(handle); }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource);
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);
        void init(struct ::wl_resource *resource);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);
        Resource *add(struct ::wl_resource *resource);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobalRemoved() const { return m_globalRemovedEvent; }
        void globalRemove();

        bool isGlobal() const { return m_global != nullptr; }
        bool isResource() const { return m_resource != nullptr; }

        static const struct ::wl_interface *interface();
        static QByteArray interfaceName() { return interface()->name; }
        static int interfaceVersion() { return interface()->version; }


        void send_delta(int32_t value120);
        void send_delta(struct ::wl_resource *resource, int32_t value120);
        void send_frame(uint32_t time);
        void send_frame(struct ::wl_resource *resource, uint32_t time);

    protected:
        virtual Resource *zwp_tablet_pad_dial_v2_allocate();

        virtual void zwp_tablet_pad_dial_v2_destroy_global();

        virtual void zwp_tablet_pad_dial_v2_bind_resource(Resource *resource);
        virtual void zwp_tablet_pad_dial_v2_destroy_resource(Resource *resource);

        virtual void zwp_tablet_pad_dial_v2_set_feedback(Resource *resource, const QString &description, uint32_t serial);
        virtual void zwp_tablet_pad_dial_v2_destroy(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);
        static void display_destroy_func(struct ::wl_listener *listener, void *data);
        static int deferred_destroy_global_func(void *data);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);
        Resource *bind(struct ::wl_resource *handle);

        static const struct ::zwp_tablet_pad_dial_v2_interface m_zwp_tablet_pad_dial_v2_interface;

        static void handle_set_feedback(
            ::wl_client *client,
            struct wl_resource *resource,
            const char *description,
            uint32_t serial);
        static void handle_destroy(
            ::wl_client *client,
            struct wl_resource *resource);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        struct ::wl_display *m_display;
        struct wl_event_source *m_globalRemovedEvent;
        struct DisplayDestroyedListener : ::wl_listener {
            zwp_tablet_pad_dial_v2 *parent;
        };
        DisplayDestroyedListener m_displayDestroyedListener;
    };
}

QT_WARNING_POP
QT_END_NAMESPACE

#endif
