// This file is generated by kconfig_compiler_kf6 from slide.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_SLIDECONFIG_H
#define KWIN_SLIDECONFIG_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class SlideConfig : public KConfigSkeleton
{
  public:

    static SlideConfig *self();
    static void instance(const QString& cfgfilename);
    static void instance(KSharedConfig::Ptr config);
    ~SlideConfig() override;

    /**
      Set HorizontalGap
    */
    static
    void setHorizontalGap( uint v )
    {
      if (!self()->isHorizontalGapImmutable())
        self()->mHorizontalGap = v;
    }

    /**
      Get HorizontalGap
    */
    static
    uint horizontalGap()
    {
      return self()->mHorizontalGap;
    }

    /**
      Is HorizontalGap Immutable
    */
    static
    bool isHorizontalGapImmutable()
    {
      return self()->isImmutable( QStringLiteral( "HorizontalGap" ) );
    }

    /**
      Set VerticalGap
    */
    static
    void setVerticalGap( uint v )
    {
      if (!self()->isVerticalGapImmutable())
        self()->mVerticalGap = v;
    }

    /**
      Get VerticalGap
    */
    static
    uint verticalGap()
    {
      return self()->mVerticalGap;
    }

    /**
      Is VerticalGap Immutable
    */
    static
    bool isVerticalGapImmutable()
    {
      return self()->isImmutable( QStringLiteral( "VerticalGap" ) );
    }

    /**
      Set SlideBackground
    */
    static
    void setSlideBackground( bool v )
    {
      if (!self()->isSlideBackgroundImmutable())
        self()->mSlideBackground = v;
    }

    /**
      Get SlideBackground
    */
    static
    bool slideBackground()
    {
      return self()->mSlideBackground;
    }

    /**
      Is SlideBackground Immutable
    */
    static
    bool isSlideBackgroundImmutable()
    {
      return self()->isImmutable( QStringLiteral( "SlideBackground" ) );
    }

  protected:
    SlideConfig(KSharedConfig::Ptr config);
    friend class SlideConfigHelper;


    // Effect-slide
    uint mHorizontalGap;
    uint mVerticalGap;
    bool mSlideBackground;
};

}

#endif

