// This file is generated by kconfig_compiler_kf6 from kwintouchscreensettings.kcfg.
// All changes you do to this file will be lost.

#include "kwintouchscreensettings.h"

using namespace KWin;

KWinTouchScreenSettings::KWinTouchScreenSettings( QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "TouchEdges" ) );

  KConfigSkeleton::ItemString  *itemTop;
  itemTop = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Top" ), mTop, QStringLiteral( "None" ) );
  addItem( itemTop, QStringLiteral( "Top" ) );
  KConfigSkeleton::ItemString  *itemRight;
  itemRight = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Right" ), mRight, QStringLiteral( "None" ) );
  addItem( itemRight, QStringLiteral( "Right" ) );
  KConfigSkeleton::ItemString  *itemBottom;
  itemBottom = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Bottom" ), mBottom, QStringLiteral( "None" ) );
  addItem( itemBottom, QStringLiteral( "Bottom" ) );
  KConfigSkeleton::ItemString  *itemLeft;
  itemLeft = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "Left" ), mLeft, QStringLiteral( "None" ) );
  addItem( itemLeft, QStringLiteral( "Left" ) );

  setCurrentGroup( QStringLiteral( "Effect-windowview" ) );

  QList<int> defaultTouchBorderActivateAll;
  defaultTouchBorderActivateAll.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivateAll;
  itemTouchBorderActivateAll = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivateAll" ), mTouchBorderActivateAll, defaultTouchBorderActivateAll );
  addItem( itemTouchBorderActivateAll, QStringLiteral( "TouchBorderActivateAll" ) );
  QList<int> defaultTouchBorderActivatePresentWindows;
  defaultTouchBorderActivatePresentWindows.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivatePresentWindows;
  itemTouchBorderActivatePresentWindows = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivate" ), mTouchBorderActivatePresentWindows, defaultTouchBorderActivatePresentWindows );
  addItem( itemTouchBorderActivatePresentWindows, QStringLiteral( "TouchBorderActivatePresentWindows" ) );
  QList<int> defaultTouchBorderActivateClass;
  defaultTouchBorderActivateClass.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivateClass;
  itemTouchBorderActivateClass = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivateClass" ), mTouchBorderActivateClass, defaultTouchBorderActivateClass );
  addItem( itemTouchBorderActivateClass, QStringLiteral( "TouchBorderActivateClass" ) );

  setCurrentGroup( QStringLiteral( "Effect-overview" ) );

  QList<int> defaultTouchBorderActivateOverview;
  defaultTouchBorderActivateOverview.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivateOverview;
  itemTouchBorderActivateOverview = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivate" ), mTouchBorderActivateOverview, defaultTouchBorderActivateOverview );
  addItem( itemTouchBorderActivateOverview, QStringLiteral( "TouchBorderActivateOverview" ) );
  QList<int> defaultTouchBorderActivateGrid;
  defaultTouchBorderActivateGrid.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivateGrid;
  itemTouchBorderActivateGrid = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "GridTouchBorderActivate" ), mTouchBorderActivateGrid, defaultTouchBorderActivateGrid );
  addItem( itemTouchBorderActivateGrid, QStringLiteral( "TouchBorderActivateGrid" ) );

  setCurrentGroup( QStringLiteral( "TabBox" ) );

  QList<int> defaultTouchBorderActivateTabBox;
  defaultTouchBorderActivateTabBox.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderActivateTabBox;
  itemTouchBorderActivateTabBox = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderActivate" ), mTouchBorderActivateTabBox, defaultTouchBorderActivateTabBox );
  addItem( itemTouchBorderActivateTabBox, QStringLiteral( "TouchBorderActivateTabBox" ) );
  QList<int> defaultTouchBorderAlternativeActivate;
  defaultTouchBorderAlternativeActivate.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemTouchBorderAlternativeActivate;
  itemTouchBorderAlternativeActivate = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "TouchBorderAlternativeActivate" ), mTouchBorderAlternativeActivate, defaultTouchBorderAlternativeActivate );
  addItem( itemTouchBorderAlternativeActivate, QStringLiteral( "TouchBorderAlternativeActivate" ) );
}

QString KWinTouchScreenSettings::defaultTopValue_helper() const
{

  return QStringLiteral( "None" );
}

QString KWinTouchScreenSettings::defaultRightValue_helper() const
{

  return QStringLiteral( "None" );
}

QString KWinTouchScreenSettings::defaultBottomValue_helper() const
{

  return QStringLiteral( "None" );
}

QString KWinTouchScreenSettings::defaultLeftValue_helper() const
{

  return QStringLiteral( "None" );
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivateAllValue_helper() const
{
  QList<int> defaultTouchBorderActivateAll;
  defaultTouchBorderActivateAll.append( ElectricNone );

  return defaultTouchBorderActivateAll;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivatePresentWindowsValue_helper() const
{
  QList<int> defaultTouchBorderActivatePresentWindows;
  defaultTouchBorderActivatePresentWindows.append( ElectricNone );

  return defaultTouchBorderActivatePresentWindows;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivateClassValue_helper() const
{
  QList<int> defaultTouchBorderActivateClass;
  defaultTouchBorderActivateClass.append( ElectricNone );

  return defaultTouchBorderActivateClass;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivateOverviewValue_helper() const
{
  QList<int> defaultTouchBorderActivateOverview;
  defaultTouchBorderActivateOverview.append( ElectricNone );

  return defaultTouchBorderActivateOverview;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivateGridValue_helper() const
{
  QList<int> defaultTouchBorderActivateGrid;
  defaultTouchBorderActivateGrid.append( ElectricNone );

  return defaultTouchBorderActivateGrid;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderActivateTabBoxValue_helper() const
{
  QList<int> defaultTouchBorderActivateTabBox;
  defaultTouchBorderActivateTabBox.append( ElectricNone );

  return defaultTouchBorderActivateTabBox;
}

QList<int> KWinTouchScreenSettings::defaultTouchBorderAlternativeActivateValue_helper() const
{
  QList<int> defaultTouchBorderAlternativeActivate;
  defaultTouchBorderAlternativeActivate.append( ElectricNone );

  return defaultTouchBorderAlternativeActivate;
}

KWinTouchScreenSettings::~KWinTouchScreenSettings()
{
}

