// This file is generated by kconfig_compiler_kf6 from kwinxwaylandsettings.kcfg.
// All changes you do to this file will be lost.

#include "kwinxwaylandsettings.h"

XwaylandSettings::XwaylandSettings( )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
{
  setCurrentGroup( QStringLiteral( "Xwayland" ) );

  QList<KConfigSkeleton::ItemEnum::Choice> valuesxwaylandEavesdrops;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("None");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Modifiers");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Combinations");
    valuesxwaylandEavesdrops.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("All");
    valuesxwaylandEavesdrops.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemXwaylandEavesdrops;
  itemXwaylandEavesdrops = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "XwaylandEavesdrops" ), mXwaylandEavesdrops, valuesxwaylandEavesdrops, EnumXwaylandEavesdrops::Combinations );
  itemXwaylandEavesdrops->setWriteFlags(KConfigBase::Notify);
  addItem( itemXwaylandEavesdrops, QStringLiteral( "xwaylandEavesdrops" ) );
  KConfigSkeleton::ItemBool  *itemXwaylandEavesdropsMouse;
  itemXwaylandEavesdropsMouse = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEavesdropsMouse" ), mXwaylandEavesdropsMouse, false );
  itemXwaylandEavesdropsMouse->setWriteFlags(KConfigBase::Notify);
  addItem( itemXwaylandEavesdropsMouse, QStringLiteral( "xwaylandEavesdropsMouse" ) );
  KConfigSkeleton::ItemBool  *itemXwaylandEisNoPrompt;
  itemXwaylandEisNoPrompt = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "XwaylandEisNoPrompt" ), mXwaylandEisNoPrompt, false );
  itemXwaylandEisNoPrompt->setWriteFlags(KConfigBase::Notify);
  addItem( itemXwaylandEisNoPrompt, QStringLiteral( "XwaylandEisNoPrompt" ) );
  KConfigSkeleton::ItemStringList  *itemXwaylandEisNoPromptApps;
  itemXwaylandEisNoPromptApps = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "XwaylandEisNoPromptApps" ), mXwaylandEisNoPromptApps );
  itemXwaylandEisNoPromptApps->setWriteFlags(KConfigBase::Notify);
  addItem( itemXwaylandEisNoPromptApps, QStringLiteral( "XwaylandEisNoPromptApps" ) );
}

XwaylandSettings::~XwaylandSettings()
{
}

