/****************************************************************************
** Meta object code from reading C++ file 'surfaceitem_wayland.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kwin-6.6.91/src/scene/surfaceitem_wayland.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'surfaceitem_wayland.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin::SurfaceItemWayland::qt_create_metaobjectdata<qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin::SurfaceItemWayland",
        "handleSurfaceCommitted",
        "",
        "handleSurfaceSizeChanged",
        "handleBufferChanged",
        "handleBufferSourceBoxChanged",
        "handleBufferTransformChanged",
        "handleChildSubSurfaceRemoved",
        "SubSurfaceInterface*",
        "child",
        "handleChildSubSurfacesChanged",
        "handleSubSurfacePositionChanged",
        "handleSurfaceMappedChanged",
        "handleColorDescriptionChanged",
        "handlePresentationModeHintChanged",
        "handleReleasePointChanged",
        "handleAlphaMultiplierChanged",
        "handleFifoFallback"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'handleSurfaceCommitted'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleSurfaceSizeChanged'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleBufferChanged'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleBufferSourceBoxChanged'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleBufferTransformChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleChildSubSurfaceRemoved'
        QtMocHelpers::SlotData<void(SubSurfaceInterface *)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'handleChildSubSurfacesChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleSubSurfacePositionChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleSurfaceMappedChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleColorDescriptionChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handlePresentationModeHintChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleReleasePointChanged'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleAlphaMultiplierChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleFifoFallback'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SurfaceItemWayland, qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin::SurfaceItemWayland::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceItem::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>.metaTypes,
    nullptr
} };

void KWin::SurfaceItemWayland::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SurfaceItemWayland *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->handleSurfaceCommitted(); break;
        case 1: _t->handleSurfaceSizeChanged(); break;
        case 2: _t->handleBufferChanged(); break;
        case 3: _t->handleBufferSourceBoxChanged(); break;
        case 4: _t->handleBufferTransformChanged(); break;
        case 5: _t->handleChildSubSurfaceRemoved((*reinterpret_cast<std::add_pointer_t<SubSurfaceInterface*>>(_a[1]))); break;
        case 6: _t->handleChildSubSurfacesChanged(); break;
        case 7: _t->handleSubSurfacePositionChanged(); break;
        case 8: _t->handleSurfaceMappedChanged(); break;
        case 9: _t->handleColorDescriptionChanged(); break;
        case 10: _t->handlePresentationModeHintChanged(); break;
        case 11: _t->handleReleasePointChanged(); break;
        case 12: _t->handleAlphaMultiplierChanged(); break;
        case 13: _t->handleFifoFallback(); break;
        default: ;
        }
    }
}

const QMetaObject *KWin::SurfaceItemWayland::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin::SurfaceItemWayland::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin18SurfaceItemWaylandE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceItem::qt_metacast(_clname);
}

int KWin::SurfaceItemWayland::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t {};
} // unnamed namespace

template <> constexpr inline auto KWin::SurfaceItemXwayland::qt_create_metaobjectdata<qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KWin::SurfaceItemXwayland"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SurfaceItemXwayland, qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KWin::SurfaceItemXwayland::staticMetaObject = { {
    QMetaObject::SuperData::link<SurfaceItemWayland::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>.metaTypes,
    nullptr
} };

void KWin::SurfaceItemXwayland::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SurfaceItemXwayland *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KWin::SurfaceItemXwayland::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KWin::SurfaceItemXwayland::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4KWin19SurfaceItemXwaylandE_t>.strings))
        return static_cast<void*>(this);
    return SurfaceItemWayland::qt_metacast(_clname);
}

int KWin::SurfaceItemXwayland::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = SurfaceItemWayland::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
