// This file is generated by kconfig_compiler_kf6 from kwinoptions_settings.kcfg.
// All changes you do to this file will be lost.

#include "kwinoptions_settings.h"

KWinOptionsSettings::KWinOptionsSettings( QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "Windows" ) );

  KConfigSkeleton::ItemInt  *itemBorderSnapZone;
  itemBorderSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "BorderSnapZone" ), mBorderSnapZone, 10 );
  itemBorderSnapZone->setMinValue(0);
  itemBorderSnapZone->setMaxValue(100);
  addItem( itemBorderSnapZone, QStringLiteral( "BorderSnapZone" ) );
  KConfigSkeleton::ItemInt  *itemWindowSnapZone;
  itemWindowSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "WindowSnapZone" ), mWindowSnapZone, 10 );
  itemWindowSnapZone->setMinValue(0);
  itemWindowSnapZone->setMaxValue(100);
  addItem( itemWindowSnapZone, QStringLiteral( "WindowSnapZone" ) );
  KConfigSkeleton::ItemInt  *itemCenterSnapZone;
  itemCenterSnapZone = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "CenterSnapZone" ), mCenterSnapZone, 0 );
  itemCenterSnapZone->setMinValue(0);
  itemCenterSnapZone->setMaxValue(100);
  addItem( itemCenterSnapZone, QStringLiteral( "CenterSnapZone" ) );
  KConfigSkeleton::ItemBool  *itemSnapOnlyWhenOverlapping;
  itemSnapOnlyWhenOverlapping = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SnapOnlyWhenOverlapping" ), mSnapOnlyWhenOverlapping, false );
  addItem( itemSnapOnlyWhenOverlapping, QStringLiteral( "SnapOnlyWhenOverlapping" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesPlacement;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Smart");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Maximizing");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Random");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Centered");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ZeroCornered");
    valuesPlacement.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("UnderMouse");
    valuesPlacement.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemPlacement;
  itemPlacement = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "Placement" ), mPlacement, valuesPlacement, Centered );
  addItem( itemPlacement, QStringLiteral( "Placement" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesActivationDesktopPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("SwitchToOtherDesktop");
    valuesActivationDesktopPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BringToCurrentDesktop");
    valuesActivationDesktopPolicy.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemActivationDesktopPolicy;
  itemActivationDesktopPolicy = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "ActivationDesktopPolicy" ), mActivationDesktopPolicy, valuesActivationDesktopPolicy, SwitchToOtherDesktop );
  addItem( itemActivationDesktopPolicy, QStringLiteral( "ActivationDesktopPolicy" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesTitlebarDoubleClickCommand;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Maximize");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeVerticalOnly");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeHorizontalOnly");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OnAllDesktops");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesTitlebarDoubleClickCommand.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemTitlebarDoubleClickCommand;
  itemTitlebarDoubleClickCommand = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "TitlebarDoubleClickCommand" ), mTitlebarDoubleClickCommand, valuesTitlebarDoubleClickCommand, EnumTitlebarDoubleClickCommand::Maximize );
  itemTitlebarDoubleClickCommand->setValueForChoice(QStringLiteral( "MaximizeVerticalOnly" ), QStringLiteral( "Maximize (vertical only)" ));
  itemTitlebarDoubleClickCommand->setValueForChoice(QStringLiteral( "MaximizeHorizontalOnly" ), QStringLiteral( "Maximize (horizontal only)" ));
  addItem( itemTitlebarDoubleClickCommand, QStringLiteral( "TitlebarDoubleClickCommand" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesMaximizeButtonLeftClickCommand;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Maximize");
    valuesMaximizeButtonLeftClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeVerticalOnly");
    valuesMaximizeButtonLeftClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeHorizontalOnly");
    valuesMaximizeButtonLeftClickCommand.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemMaximizeButtonLeftClickCommand;
  itemMaximizeButtonLeftClickCommand = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "MaximizeButtonLeftClickCommand" ), mMaximizeButtonLeftClickCommand, valuesMaximizeButtonLeftClickCommand, EnumMaximizeButtonLeftClickCommand::Maximize );
  itemMaximizeButtonLeftClickCommand->setValueForChoice(QStringLiteral( "MaximizeVerticalOnly" ), QStringLiteral( "Maximize (vertical only)" ));
  itemMaximizeButtonLeftClickCommand->setValueForChoice(QStringLiteral( "MaximizeHorizontalOnly" ), QStringLiteral( "Maximize (horizontal only)" ));
  addItem( itemMaximizeButtonLeftClickCommand, QStringLiteral( "MaximizeButtonLeftClickCommand" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesMaximizeButtonMiddleClickCommand;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Maximize");
    valuesMaximizeButtonMiddleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeVerticalOnly");
    valuesMaximizeButtonMiddleClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeHorizontalOnly");
    valuesMaximizeButtonMiddleClickCommand.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemMaximizeButtonMiddleClickCommand;
  itemMaximizeButtonMiddleClickCommand = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "MaximizeButtonMiddleClickCommand" ), mMaximizeButtonMiddleClickCommand, valuesMaximizeButtonMiddleClickCommand, EnumMaximizeButtonMiddleClickCommand::MaximizeVerticalOnly );
  itemMaximizeButtonMiddleClickCommand->setValueForChoice(QStringLiteral( "MaximizeVerticalOnly" ), QStringLiteral( "Maximize (vertical only)" ));
  itemMaximizeButtonMiddleClickCommand->setValueForChoice(QStringLiteral( "MaximizeHorizontalOnly" ), QStringLiteral( "Maximize (horizontal only)" ));
  addItem( itemMaximizeButtonMiddleClickCommand, QStringLiteral( "MaximizeButtonMiddleClickCommand" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesMaximizeButtonRightClickCommand;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Maximize");
    valuesMaximizeButtonRightClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeVerticalOnly");
    valuesMaximizeButtonRightClickCommand.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaximizeHorizontalOnly");
    valuesMaximizeButtonRightClickCommand.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemMaximizeButtonRightClickCommand;
  itemMaximizeButtonRightClickCommand = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "MaximizeButtonRightClickCommand" ), mMaximizeButtonRightClickCommand, valuesMaximizeButtonRightClickCommand, EnumMaximizeButtonRightClickCommand::MaximizeHorizontalOnly );
  itemMaximizeButtonRightClickCommand->setValueForChoice(QStringLiteral( "MaximizeVerticalOnly" ), QStringLiteral( "Maximize (vertical only)" ));
  itemMaximizeButtonRightClickCommand->setValueForChoice(QStringLiteral( "MaximizeHorizontalOnly" ), QStringLiteral( "Maximize (horizontal only)" ));
  addItem( itemMaximizeButtonRightClickCommand, QStringLiteral( "MaximizeButtonRightClickCommand" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesFocusPolicy;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ClickToFocus");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusFollowsMouse");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusUnderMouse");
    valuesFocusPolicy.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("FocusStrictlyUnderMouse");
    valuesFocusPolicy.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemFocusPolicy;
  itemFocusPolicy = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "FocusPolicy" ), mFocusPolicy, valuesFocusPolicy, EnumFocusPolicy::ClickToFocus );
  addItem( itemFocusPolicy, QStringLiteral( "FocusPolicy" ) );
  KConfigSkeleton::ItemBool  *itemNextFocusPrefersMouse;
  itemNextFocusPrefersMouse = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "NextFocusPrefersMouse" ), mNextFocusPrefersMouse, false );
  addItem( itemNextFocusPrefersMouse, QStringLiteral( "NextFocusPrefersMouse" ) );
  KConfigSkeleton::ItemInt  *itemAutoRaiseInterval;
  itemAutoRaiseInterval = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "AutoRaiseInterval" ), mAutoRaiseInterval, 750 );
  itemAutoRaiseInterval->setMinValue(0);
  addItem( itemAutoRaiseInterval, QStringLiteral( "AutoRaiseInterval" ) );
  KConfigSkeleton::ItemInt  *itemDelayFocusInterval;
  itemDelayFocusInterval = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "DelayFocusInterval" ), mDelayFocusInterval, 300 );
  itemDelayFocusInterval->setMinValue(0);
  addItem( itemDelayFocusInterval, QStringLiteral( "DelayFocusInterval" ) );
  KConfigSkeleton::ItemBool  *itemAutoRaise;
  itemAutoRaise = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AutoRaise" ), mAutoRaise, false );
  addItem( itemAutoRaise, QStringLiteral( "AutoRaise" ) );
  KConfigSkeleton::ItemBool  *itemClickRaise;
  itemClickRaise = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ClickRaise" ), mClickRaise, true );
  addItem( itemClickRaise, QStringLiteral( "ClickRaise" ) );
  KConfigSkeleton::ItemBool  *itemSeparateScreenFocus;
  itemSeparateScreenFocus = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SeparateScreenFocus" ), mSeparateScreenFocus, true );
  addItem( itemSeparateScreenFocus, QStringLiteral( "SeparateScreenFocus" ) );
  KConfigSkeleton::ItemInt  *itemFocusStealingPreventionLevel;
  itemFocusStealingPreventionLevel = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "FocusStealingPreventionLevel" ), mFocusStealingPreventionLevel, 1 );
  itemFocusStealingPreventionLevel->setMinValue(0);
  itemFocusStealingPreventionLevel->setMaxValue(4);
  addItem( itemFocusStealingPreventionLevel, QStringLiteral( "FocusStealingPreventionLevel" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesPictureInPictureHomeCorner;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TopLeftCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("TopRightCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BottomLeftCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("BottomRightCorner");
    valuesPictureInPictureHomeCorner.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemPictureInPictureHomeCorner;
  itemPictureInPictureHomeCorner = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "PictureInPictureHomeCorner" ), mPictureInPictureHomeCorner, valuesPictureInPictureHomeCorner, Qt::BottomRightCorner );
  addItem( itemPictureInPictureHomeCorner, QStringLiteral( "PictureInPictureHomeCorner" ) );
  KConfigSkeleton::ItemInt  *itemPictureInPictureMargin;
  itemPictureInPictureMargin = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "PictureInPictureMargin" ), mPictureInPictureMargin, 20 );
  addItem( itemPictureInPictureMargin, QStringLiteral( "PictureInPictureMargin" ) );

  setCurrentGroup( QStringLiteral( "MouseBindings" ) );

  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandActiveTitlebar1;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandActiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandActiveTitlebar1.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandActiveTitlebar1;
  itemCommandActiveTitlebar1 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandActiveTitlebar1" ), mCommandActiveTitlebar1, valuesCommandActiveTitlebar1, EnumCommandActiveTitlebar1::Raise );
  itemCommandActiveTitlebar1->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandActiveTitlebar1->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandActiveTitlebar1, QStringLiteral( "CommandActiveTitlebar1" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandActiveTitlebar2;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandActiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandActiveTitlebar2.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandActiveTitlebar2;
  itemCommandActiveTitlebar2 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandActiveTitlebar2" ), mCommandActiveTitlebar2, valuesCommandActiveTitlebar2, EnumCommandActiveTitlebar2::Nothing );
  itemCommandActiveTitlebar2->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandActiveTitlebar2->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandActiveTitlebar2, QStringLiteral( "CommandActiveTitlebar2" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandActiveTitlebar3;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandActiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandActiveTitlebar3.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandActiveTitlebar3;
  itemCommandActiveTitlebar3 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandActiveTitlebar3" ), mCommandActiveTitlebar3, valuesCommandActiveTitlebar3, EnumCommandActiveTitlebar3::OperationsMenu );
  itemCommandActiveTitlebar3->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandActiveTitlebar3->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandActiveTitlebar3, QStringLiteral( "CommandActiveTitlebar3" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandTitlebarWheel;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("RaiseLower");
    valuesCommandTitlebarWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaxmizeRestore");
    valuesCommandTitlebarWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("AboveBelow");
    valuesCommandTitlebarWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PreviousNextDesktop");
    valuesCommandTitlebarWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ChangeOpacity");
    valuesCommandTitlebarWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandTitlebarWheel.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandTitlebarWheel;
  itemCommandTitlebarWheel = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandTitlebarWheel" ), mCommandTitlebarWheel, valuesCommandTitlebarWheel, EnumCommandTitlebarWheel::Nothing );
  itemCommandTitlebarWheel->setValueForChoice(QStringLiteral( "RaiseLower" ), QStringLiteral( "Raise/Lower" ));
  itemCommandTitlebarWheel->setValueForChoice(QStringLiteral( "MaxmizeRestore" ), QStringLiteral( "Maximize/Restore" ));
  itemCommandTitlebarWheel->setValueForChoice(QStringLiteral( "AboveBelow" ), QStringLiteral( "Above/Below" ));
  itemCommandTitlebarWheel->setValueForChoice(QStringLiteral( "PreviousNextDesktop" ), QStringLiteral( "Previous/Next Desktop" ));
  itemCommandTitlebarWheel->setValueForChoice(QStringLiteral( "ChangeOpacity" ), QStringLiteral( "Change Opacity" ));
  addItem( itemCommandTitlebarWheel, QStringLiteral( "CommandTitlebarWheel" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandInactiveTitlebar1;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndRaise");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndLower");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandInactiveTitlebar1.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandInactiveTitlebar1;
  itemCommandInactiveTitlebar1 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandInactiveTitlebar1" ), mCommandInactiveTitlebar1, valuesCommandInactiveTitlebar1, EnumCommandInactiveTitlebar1::ActivateAndRaise );
  itemCommandInactiveTitlebar1->setValueForChoice(QStringLiteral( "ActivateAndRaise" ), QStringLiteral( "Activate and raise" ));
  itemCommandInactiveTitlebar1->setValueForChoice(QStringLiteral( "ActivateAndLower" ), QStringLiteral( "Activate and lower" ));
  itemCommandInactiveTitlebar1->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandInactiveTitlebar1->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandInactiveTitlebar1, QStringLiteral( "CommandInactiveTitlebar1" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandInactiveTitlebar2;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndRaise");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndLower");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandInactiveTitlebar2.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandInactiveTitlebar2;
  itemCommandInactiveTitlebar2 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandInactiveTitlebar2" ), mCommandInactiveTitlebar2, valuesCommandInactiveTitlebar2, EnumCommandInactiveTitlebar2::Nothing );
  itemCommandInactiveTitlebar2->setValueForChoice(QStringLiteral( "ActivateAndRaise" ), QStringLiteral( "Activate and raise" ));
  itemCommandInactiveTitlebar2->setValueForChoice(QStringLiteral( "ActivateAndLower" ), QStringLiteral( "Activate and lower" ));
  itemCommandInactiveTitlebar2->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandInactiveTitlebar2->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandInactiveTitlebar2, QStringLiteral( "CommandInactiveTitlebar2" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandInactiveTitlebar3;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndRaise");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateAndLower");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Close");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandInactiveTitlebar3.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandInactiveTitlebar3;
  itemCommandInactiveTitlebar3 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandInactiveTitlebar3" ), mCommandInactiveTitlebar3, valuesCommandInactiveTitlebar3, EnumCommandInactiveTitlebar3::OperationsMenu );
  itemCommandInactiveTitlebar3->setValueForChoice(QStringLiteral( "ActivateAndRaise" ), QStringLiteral( "Activate and raise" ));
  itemCommandInactiveTitlebar3->setValueForChoice(QStringLiteral( "ActivateAndLower" ), QStringLiteral( "Activate and lower" ));
  itemCommandInactiveTitlebar3->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandInactiveTitlebar3->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandInactiveTitlebar3, QStringLiteral( "CommandInactiveTitlebar3" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandWindow1;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaiseOnReleaseAndPassClick");
    valuesCommandWindow1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaisePassClick");
    valuesCommandWindow1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivatePassClick");
    valuesCommandWindow1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandWindow1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaise");
    valuesCommandWindow1.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandWindow1;
  itemCommandWindow1 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandWindow1" ), mCommandWindow1, valuesCommandWindow1, EnumCommandWindow1::ActivateRaiseOnReleaseAndPassClick );
  itemCommandWindow1->setValueForChoice(QStringLiteral( "ActivateRaiseOnReleaseAndPassClick" ), QStringLiteral( "Activate, pass click and raise on release" ));
  itemCommandWindow1->setValueForChoice(QStringLiteral( "ActivateRaisePassClick" ), QStringLiteral( "Activate, raise and pass click" ));
  itemCommandWindow1->setValueForChoice(QStringLiteral( "ActivatePassClick" ), QStringLiteral( "Activate and pass click" ));
  itemCommandWindow1->setValueForChoice(QStringLiteral( "ActivateRaise" ), QStringLiteral( "Activate and raise" ));
  addItem( itemCommandWindow1, QStringLiteral( "CommandWindow1" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandWindow2;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaisePassClick");
    valuesCommandWindow2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivatePassClick");
    valuesCommandWindow2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandWindow2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaise");
    valuesCommandWindow2.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandWindow2;
  itemCommandWindow2 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandWindow2" ), mCommandWindow2, valuesCommandWindow2, EnumCommandWindow2::ActivatePassClick );
  itemCommandWindow2->setValueForChoice(QStringLiteral( "ActivateRaisePassClick" ), QStringLiteral( "Activate, raise and pass click" ));
  itemCommandWindow2->setValueForChoice(QStringLiteral( "ActivatePassClick" ), QStringLiteral( "Activate and pass click" ));
  itemCommandWindow2->setValueForChoice(QStringLiteral( "ActivateRaise" ), QStringLiteral( "Activate and raise" ));
  addItem( itemCommandWindow2, QStringLiteral( "CommandWindow2" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandWindow3;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaisePassClick");
    valuesCommandWindow3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivatePassClick");
    valuesCommandWindow3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Activate");
    valuesCommandWindow3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaise");
    valuesCommandWindow3.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandWindow3;
  itemCommandWindow3 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandWindow3" ), mCommandWindow3, valuesCommandWindow3, EnumCommandWindow3::ActivatePassClick );
  itemCommandWindow3->setValueForChoice(QStringLiteral( "ActivateRaisePassClick" ), QStringLiteral( "Activate, raise and pass click" ));
  itemCommandWindow3->setValueForChoice(QStringLiteral( "ActivatePassClick" ), QStringLiteral( "Activate and pass click" ));
  itemCommandWindow3->setValueForChoice(QStringLiteral( "ActivateRaise" ), QStringLiteral( "Activate and raise" ));
  addItem( itemCommandWindow3, QStringLiteral( "CommandWindow3" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandWindowWheel;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Scroll");
    valuesCommandWindowWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateScroll");
    valuesCommandWindowWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaiseAndScroll");
    valuesCommandWindowWheel.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandWindowWheel;
  itemCommandWindowWheel = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandWindowWheel" ), mCommandWindowWheel, valuesCommandWindowWheel, EnumCommandWindowWheel::Scroll );
  itemCommandWindowWheel->setValueForChoice(QStringLiteral( "ActivateScroll" ), QStringLiteral( "Activate and scroll" ));
  itemCommandWindowWheel->setValueForChoice(QStringLiteral( "ActivateRaiseAndScroll" ), QStringLiteral( "Activate, raise and scroll" ));
  addItem( itemCommandWindowWheel, QStringLiteral( "CommandWindowWheel" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandAllKey;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Meta");
    valuesCommandAllKey.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Alt");
    valuesCommandAllKey.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandAllKey;
  itemCommandAllKey = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandAllKey" ), mCommandAllKey, valuesCommandAllKey, EnumCommandAllKey::Meta );
  addItem( itemCommandAllKey, QStringLiteral( "CommandAllKey" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandAll1;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Move");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaiseAndMove");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Resize");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("DecreaseOpacity");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("IncreaseOpactiy");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandAll1.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandAll1.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandAll1;
  itemCommandAll1 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandAll1" ), mCommandAll1, valuesCommandAll1, EnumCommandAll1::Move );
  itemCommandAll1->setValueForChoice(QStringLiteral( "ActivateRaiseAndMove" ), QStringLiteral( "Activate, raise and move" ));
  itemCommandAll1->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandAll1->setValueForChoice(QStringLiteral( "DecreaseOpacity" ), QStringLiteral( "Decrease Opacity" ));
  itemCommandAll1->setValueForChoice(QStringLiteral( "IncreaseOpactiy" ), QStringLiteral( "Increase Opacity" ));
  itemCommandAll1->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandAll1, QStringLiteral( "CommandAll1" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandAll2;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Move");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaiseAndMove");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Resize");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("DecreaseOpacity");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("IncreaseOpactiy");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandAll2.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandAll2.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandAll2;
  itemCommandAll2 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandAll2" ), mCommandAll2, valuesCommandAll2, EnumCommandAll2::ToggleRaiseAndLower );
  itemCommandAll2->setValueForChoice(QStringLiteral( "ActivateRaiseAndMove" ), QStringLiteral( "Activate, raise and move" ));
  itemCommandAll2->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandAll2->setValueForChoice(QStringLiteral( "DecreaseOpacity" ), QStringLiteral( "Decrease Opacity" ));
  itemCommandAll2->setValueForChoice(QStringLiteral( "IncreaseOpactiy" ), QStringLiteral( "Increase Opacity" ));
  itemCommandAll2->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandAll2, QStringLiteral( "CommandAll2" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandAll3;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Move");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ActivateRaiseAndMove");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ToggleRaiseAndLower");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Resize");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Raise");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Lower");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Minimize");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("DecreaseOpacity");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("IncreaseOpactiy");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("OperationsMenu");
    valuesCommandAll3.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandAll3.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandAll3;
  itemCommandAll3 = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandAll3" ), mCommandAll3, valuesCommandAll3, EnumCommandAll3::Resize );
  itemCommandAll3->setValueForChoice(QStringLiteral( "ActivateRaiseAndMove" ), QStringLiteral( "Activate, raise and move" ));
  itemCommandAll3->setValueForChoice(QStringLiteral( "ToggleRaiseAndLower" ), QStringLiteral( "Toggle raise and lower" ));
  itemCommandAll3->setValueForChoice(QStringLiteral( "DecreaseOpacity" ), QStringLiteral( "Decrease Opacity" ));
  itemCommandAll3->setValueForChoice(QStringLiteral( "IncreaseOpactiy" ), QStringLiteral( "Increase Opacity" ));
  itemCommandAll3->setValueForChoice(QStringLiteral( "OperationsMenu" ), QStringLiteral( "Operations menu" ));
  addItem( itemCommandAll3, QStringLiteral( "CommandAll3" ) );
  QList<KConfigSkeleton::ItemEnum::Choice> valuesCommandAllWheel;
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("RaiseLower");
    valuesCommandAllWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("MaxmizeRestore");
    valuesCommandAllWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("AboveBelow");
    valuesCommandAllWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("PreviousNextDesktop");
    valuesCommandAllWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("ChangeOpacity");
    valuesCommandAllWheel.append( choice );
  }
  {
    KConfigSkeleton::ItemEnum::Choice choice;
    choice.name = QStringLiteral("Nothing");
    valuesCommandAllWheel.append( choice );
  }
  KConfigSkeleton::ItemEnum  *itemCommandAllWheel;
  itemCommandAllWheel = new KConfigSkeleton::ItemEnum( currentGroup(), QStringLiteral( "CommandAllWheel" ), mCommandAllWheel, valuesCommandAllWheel, EnumCommandAllWheel::Nothing );
  itemCommandAllWheel->setValueForChoice(QStringLiteral( "RaiseLower" ), QStringLiteral( "Raise/Lower" ));
  itemCommandAllWheel->setValueForChoice(QStringLiteral( "MaxmizeRestore" ), QStringLiteral( "Maximize/Restore" ));
  itemCommandAllWheel->setValueForChoice(QStringLiteral( "AboveBelow" ), QStringLiteral( "Above/Below" ));
  itemCommandAllWheel->setValueForChoice(QStringLiteral( "PreviousNextDesktop" ), QStringLiteral( "Previous/Next Desktop" ));
  itemCommandAllWheel->setValueForChoice(QStringLiteral( "ChangeOpacity" ), QStringLiteral( "Change Opacity" ));
  addItem( itemCommandAllWheel, QStringLiteral( "CommandAllWheel" ) );
  KConfigSkeleton::ItemBool  *itemDoubleClickBorderToMaximize;
  itemDoubleClickBorderToMaximize = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "DoubleClickBorderToMaximize" ), mDoubleClickBorderToMaximize, true );
  addItem( itemDoubleClickBorderToMaximize, QStringLiteral( "DoubleClickBorderToMaximize" ) );
}

int KWinOptionsSettings::defaultBorderSnapZoneValue_helper() const
{

  return 10;
}

int KWinOptionsSettings::defaultWindowSnapZoneValue_helper() const
{

  return 10;
}

int KWinOptionsSettings::defaultCenterSnapZoneValue_helper() const
{

  return 0;
}

bool KWinOptionsSettings::defaultSnapOnlyWhenOverlappingValue_helper() const
{

  return false;
}

int KWinOptionsSettings::defaultPlacementValue_helper() const
{

  return Centered;
}

int KWinOptionsSettings::defaultActivationDesktopPolicyValue_helper() const
{

  return SwitchToOtherDesktop;
}

int KWinOptionsSettings::defaultTitlebarDoubleClickCommandValue_helper() const
{

  return EnumTitlebarDoubleClickCommand::Maximize;
}

int KWinOptionsSettings::defaultMaximizeButtonLeftClickCommandValue_helper() const
{

  return EnumMaximizeButtonLeftClickCommand::Maximize;
}

int KWinOptionsSettings::defaultMaximizeButtonMiddleClickCommandValue_helper() const
{

  return EnumMaximizeButtonMiddleClickCommand::MaximizeVerticalOnly;
}

int KWinOptionsSettings::defaultMaximizeButtonRightClickCommandValue_helper() const
{

  return EnumMaximizeButtonRightClickCommand::MaximizeHorizontalOnly;
}

int KWinOptionsSettings::defaultFocusPolicyValue_helper() const
{

  return EnumFocusPolicy::ClickToFocus;
}

bool KWinOptionsSettings::defaultNextFocusPrefersMouseValue_helper() const
{

  return false;
}

int KWinOptionsSettings::defaultAutoRaiseIntervalValue_helper() const
{

  return 750;
}

int KWinOptionsSettings::defaultDelayFocusIntervalValue_helper() const
{

  return 300;
}

bool KWinOptionsSettings::defaultAutoRaiseValue_helper() const
{

  return false;
}

bool KWinOptionsSettings::defaultClickRaiseValue_helper() const
{

  return true;
}

bool KWinOptionsSettings::defaultSeparateScreenFocusValue_helper() const
{

  return true;
}

int KWinOptionsSettings::defaultFocusStealingPreventionLevelValue_helper() const
{

  return 1;
}

int KWinOptionsSettings::defaultPictureInPictureHomeCornerValue_helper() const
{

  return Qt::BottomRightCorner;
}

int KWinOptionsSettings::defaultPictureInPictureMarginValue_helper() const
{

  return 20;
}

int KWinOptionsSettings::defaultCommandActiveTitlebar1Value_helper() const
{

  return EnumCommandActiveTitlebar1::Raise;
}

int KWinOptionsSettings::defaultCommandActiveTitlebar2Value_helper() const
{

  return EnumCommandActiveTitlebar2::Nothing;
}

int KWinOptionsSettings::defaultCommandActiveTitlebar3Value_helper() const
{

  return EnumCommandActiveTitlebar3::OperationsMenu;
}

int KWinOptionsSettings::defaultCommandTitlebarWheelValue_helper() const
{

  return EnumCommandTitlebarWheel::Nothing;
}

int KWinOptionsSettings::defaultCommandInactiveTitlebar1Value_helper() const
{

  return EnumCommandInactiveTitlebar1::ActivateAndRaise;
}

int KWinOptionsSettings::defaultCommandInactiveTitlebar2Value_helper() const
{

  return EnumCommandInactiveTitlebar2::Nothing;
}

int KWinOptionsSettings::defaultCommandInactiveTitlebar3Value_helper() const
{

  return EnumCommandInactiveTitlebar3::OperationsMenu;
}

int KWinOptionsSettings::defaultCommandWindow1Value_helper() const
{

  return EnumCommandWindow1::ActivateRaiseOnReleaseAndPassClick;
}

int KWinOptionsSettings::defaultCommandWindow2Value_helper() const
{

  return EnumCommandWindow2::ActivatePassClick;
}

int KWinOptionsSettings::defaultCommandWindow3Value_helper() const
{

  return EnumCommandWindow3::ActivatePassClick;
}

int KWinOptionsSettings::defaultCommandWindowWheelValue_helper() const
{

  return EnumCommandWindowWheel::Scroll;
}

int KWinOptionsSettings::defaultCommandAllKeyValue_helper() const
{

  return EnumCommandAllKey::Meta;
}

int KWinOptionsSettings::defaultCommandAll1Value_helper() const
{

  return EnumCommandAll1::Move;
}

int KWinOptionsSettings::defaultCommandAll2Value_helper() const
{

  return EnumCommandAll2::ToggleRaiseAndLower;
}

int KWinOptionsSettings::defaultCommandAll3Value_helper() const
{

  return EnumCommandAll3::Resize;
}

int KWinOptionsSettings::defaultCommandAllWheelValue_helper() const
{

  return EnumCommandAllWheel::Nothing;
}

bool KWinOptionsSettings::defaultDoubleClickBorderToMaximizeValue_helper() const
{

  return true;
}

KWinOptionsSettings::~KWinOptionsSettings()
{
}

