/*
    SPDX-FileCopyrightText: 2026 Vlad Zahorodnii <vlad.zahorodnii@kde.org>

    SPDX-License-Identifier: GPL-2.0-or-later
*/

#include "scene/atlas.h"

namespace KWin
{

Atlas::Atlas()
{
}

Atlas::~Atlas()
{
}

} // namespace KWin
