// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#include "virtualkeyboarddata.h"

#include "virtualkeyboardsettings.h"




VirtualKeyboardData::VirtualKeyboardData(QObject *parent)
: KCModuleData(parent)
, m_settings(new VirtualKeyboardSettings(this))

{
autoRegisterSkeletons();
}

VirtualKeyboardSettings *VirtualKeyboardData::settings() const
{
    return m_settings;
}





#include "moc_virtualkeyboarddata.cpp"
