// This file is generated by kconfig_compiler_kf6 from kwinscreenedgescriptsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_KWINSCREENEDGESCRIPTSETTINGS_H
#define KWIN_KWINSCREENEDGESCRIPTSETTINGS_H

#include "effect/globals.h"

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {

class KWinScreenEdgeScriptSettings : public KConfigSkeleton
{
  public:

    KWinScreenEdgeScriptSettings( const QString & ScriptName, QObject *parent = nullptr );
    ~KWinScreenEdgeScriptSettings() override;

    /**
      Set BorderActivate
    */
    void setBorderActivate( const QList<int> & v )
    {
      if (!isBorderActivateImmutable())
        mBorderActivate = v;
    }

    /**
      Get BorderActivate
    */
    QList<int> borderActivate() const
    {
      return mBorderActivate;
    }

    /**
      Is BorderActivate Immutable
    */
    bool isBorderActivateImmutable() const
    {
      return isImmutable( QStringLiteral( "BorderActivate" ) );
    }

    /**
      Get BorderActivate default value
    */
    QList<int> defaultBorderActivateValue() const
    {
        return defaultBorderActivateValue_helper();
    }

  protected:
    QString mParamScriptName;

    // Script-$(ScriptName)
    QList<int> mBorderActivate;
    QList<int> defaultBorderActivateValue_helper() const;
};

}

#endif

