// This file was generated by kcmutils_generate_module_data(): DO NOT EDIT!

#include "kwinxwaylanddata.h"

#include "kwinxwaylandsettings.h"




KWinXwaylandData::KWinXwaylandData(QObject *parent)
: KCModuleData(parent)
, m_settings(new KWinXwaylandSettings(this))

{
autoRegisterSkeletons();
}

KWinXwaylandSettings *KWinXwaylandData::settings() const
{
    return m_settings;
}





#include "moc_kwinxwaylanddata.cpp"
