// This file is generated by kconfig_compiler_kf6 from kwinscreenedgescriptsettings.kcfg.
// All changes you do to this file will be lost.

#include "kwinscreenedgescriptsettings.h"

using namespace KWin;

KWinScreenEdgeScriptSettings::KWinScreenEdgeScriptSettings( const QString & ScriptName, QObject *parent )
  : KConfigSkeleton( QStringLiteral( "kwinrc" ) )
  , mParamScriptName(ScriptName)
{
  setParent(parent);
  setCurrentGroup( QStringLiteral( "Script-%1" ).arg( mParamScriptName ) );

  QList<int> defaultBorderActivate;
  defaultBorderActivate.append( ElectricNone );

  KConfigSkeleton::ItemIntList  *itemBorderActivate;
  itemBorderActivate = new KConfigSkeleton::ItemIntList( currentGroup(), QStringLiteral( "BorderActivate" ), mBorderActivate, defaultBorderActivate );
  addItem( itemBorderActivate, QStringLiteral( "BorderActivate" ) );
}

QList<int> KWinScreenEdgeScriptSettings::defaultBorderActivateValue_helper() const
{
  QList<int> defaultBorderActivate;
  defaultBorderActivate.append( ElectricNone );

  return defaultBorderActivate;
}

KWinScreenEdgeScriptSettings::~KWinScreenEdgeScriptSettings()
{
}

