/****************************************************************************
** Meta object code from reading C++ file 'compositingadaptor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "compositingadaptor.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'compositingadaptor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18CompositingAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto CompositingAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18CompositingAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CompositingAdaptor",
        "D-Bus Interface",
        "org.kde.kwin.Compositing",
        "D-Bus Introspection",
        "  <interface name=\"org.kde.kwin.Compositing\">\n    <property acc"
        "ess=\"read\" type=\"b\" name=\"active\"/>\n    <property access=\""
        "read\" type=\"b\" name=\"compositingPossible\"/>\n    <property ac"
        "cess=\"read\" type=\"s\" name=\"compositingNotPossibleReason\"/>\n"
        "    <property access=\"read\" type=\"b\" name=\"openGLIsBroken\"/>"
        "\n    <property access=\"read\" type=\"s\" name=\"compositingType\""
        "/>\n    <property access=\"read\" type=\"as\" name=\"supportedOpen"
        "GLPlatformInterfaces\"/>\n    <property access=\"read\" type=\"b\""
        " name=\"platformRequiresCompositing\"/>\n    <signal name=\"compos"
        "itingToggled\">\n      <arg direction=\"out\" type=\"b\" name=\"ac"
        "tive\"/>\n    </signal>\n    <method name=\"reinitialize\"/>\n  </"
        "interface>\n",
        "compositingToggled",
        "",
        "active",
        "reinitialize",
        "compositingNotPossibleReason",
        "compositingPossible",
        "compositingType",
        "openGLIsBroken",
        "platformRequiresCompositing",
        "supportedOpenGLPlatformInterfaces"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'compositingToggled'
        QtMocHelpers::SignalData<void(bool)>(5, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 7 },
        }}),
        // Slot 'reinitialize'
        QtMocHelpers::SlotData<void()>(8, 6, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'active'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compositingNotPossibleReason'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'compositingPossible'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'compositingType'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags),
        // property 'openGLIsBroken'
        QtMocHelpers::PropertyData<bool>(12, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'platformRequiresCompositing'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags),
        // property 'supportedOpenGLPlatformInterfaces'
        QtMocHelpers::PropertyData<QStringList>(14, QMetaType::QStringList, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<CompositingAdaptor, qt_meta_tag_ZN18CompositingAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject CompositingAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CompositingAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CompositingAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18CompositingAdaptorE_t>.metaTypes,
    nullptr
} };

void CompositingAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CompositingAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->compositingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->reinitialize(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CompositingAdaptor::*)(bool )>(_a, &CompositingAdaptor::compositingToggled, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->active(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->compositingNotPossibleReason(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->compositingPossible(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->compositingType(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->openGLIsBroken(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->platformRequiresCompositing(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->supportedOpenGLPlatformInterfaces(); break;
        default: break;
        }
    }
}

const QMetaObject *CompositingAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CompositingAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CompositingAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int CompositingAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void CompositingAdaptor::compositingToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
