// This file is generated by kconfig_compiler_kf6 from kwinswitcheffectsettings.kcfg.
// All changes you do to this file will be lost.
#ifndef KWIN_TABBOX_SWITCHEFFECTSETTINGS_H
#define KWIN_TABBOX_SWITCHEFFECTSETTINGS_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

namespace KWin {
namespace TabBox {

class SwitchEffectSettings : public KConfigSkeleton
{
  public:

    SwitchEffectSettings( const QString & EffectGroup, QObject *parent = nullptr );
    ~SwitchEffectSettings() override;

    /**
      Set TabBox
    */
    void setTabBox( bool v )
    {
      if (!isTabBoxImmutable())
        mTabBox = v;
    }

    /**
      Get TabBox
    */
    bool tabBox() const
    {
      return mTabBox;
    }

    /**
      Is TabBox Immutable
    */
    bool isTabBoxImmutable() const
    {
      return isImmutable( QStringLiteral( "TabBox" ) );
    }

    /**
      Get TabBox default value
    */
    bool defaultTabBoxValue() const
    {
        return defaultTabBoxValue_helper();
    }

    /**
      Set TabBoxAlternative
    */
    void setTabBoxAlternative( bool v )
    {
      if (!isTabBoxAlternativeImmutable())
        mTabBoxAlternative = v;
    }

    /**
      Get TabBoxAlternative
    */
    bool tabBoxAlternative() const
    {
      return mTabBoxAlternative;
    }

    /**
      Is TabBoxAlternative Immutable
    */
    bool isTabBoxAlternativeImmutable() const
    {
      return isImmutable( QStringLiteral( "TabBoxAlternative" ) );
    }

    /**
      Get TabBoxAlternative default value
    */
    bool defaultTabBoxAlternativeValue() const
    {
        return defaultTabBoxAlternativeValue_helper();
    }

  protected:
    QString mParamEffectGroup;

    // $(EffectGroup)
    bool mTabBox;
    bool defaultTabBoxValue_helper() const;
    bool mTabBoxAlternative;
    bool defaultTabBoxAlternativeValue_helper() const;
};

}
}

#endif

